#!/user/bin/bash

#à partir de x fichiers txt contenant chacun une liste d'URLs, génère un fichier html avec x tableaux.

#le programme s'utilise comme ceci :
#bash ./PROGRAMMES/tableaux_URLs.sh ./URLS ./TABLEAUX/tableaux_URLs.html "anglicisme|anglicismes|anglicismo|anglicismi|anglismo|anglismi|inglesismo|inglesismi|الدخيل|الانجليزي|نكلزة"

#premier agument du programme : input : $1 le répertoire URLS qui contient tous les fichiers d'URLs .txt
dossierURL=$1;
#deuxième argument du programme : output : $2 : le nom du tableau html
monTableau=$2;
#troisième argument du programme : motif : le mot pour lequel on recherche les occurrences
motif=$3;

#écriture du début du fichier HTML
echo "<html><head><meta charset=\"UTF-8\"/><title>TABLEAUX URLS</title></head><body>" > $monTableau;
	
#première boucle : pour chaque fichier d'URL
compteurTableau=1; #déclaration-initialisation de la variable compteurTableau qui va compter le nb de fichiers
for fichier in $(ls $dossierURL)
	do
		#On crée un nouveau tableau pour chaque fichier d'URLs
		echo "<table align=center border=1 width=100%>" >> $monTableau #début du tableau
		echo "<tr bgcolor = #3CA9D1><th colspan=\"11\">TABLEAU NUMERO $compteurTableau</th></tr>" >> $monTableau #titre du tableau
		#1ère ligne du tableau avec les intitulés de nos colonnes
		echo "<tr bgcolor = #84E3F7><td>NUMERO URL</td>
		<td>HTTP</td>
		<td>ENCODAGE</d>
		<td>URL</td>
		<td>PAGE ASPIREE</td>
		<td>DUMP</td>
		<td>OCCURRENCES MOTIF</td>
		<td>CONTEXTES TXT</td>
		<td>CONTEXTES HTML</td>
		<td>INDEX HIERARCHIQUE</td>
		<td>BIGRAMMES</td></tr>" >> $monTableau;
		
		#deuxième boucle : lecture du fichier ligne par ligne
		compteur=1; #déclaration-initialisation de la variable compteur
		#pour chaque ligne du fichier d'URLS, càd pour chaque URL, on crée une ligne dans le tableau
		while read ligne
			do
				#on veut aspirer les URLS et on les sauvegarde dans le répertoire PAGES-ASPIREES
				#d'abord, il faut s'assurer que la commande curl se passe bien
				#on crée une variable codeHttp qui va récupérer le code retour fourni par http_code généré par curl
				#on utilise -s et -L pour suivre les redirections
				codeHttp=$(curl $ligne -s -L -o ./PAGES-ASPIREES/"$compteurTableau-$compteur".html -w %{http_code});
				
				#on teste la valeur de codeHttp
				#si cette valeur est 200 : tout se passe bien. Sinon, on ne peut pas récupérer la page aspirée
				if [[ $codeHttp == 200 ]]
					then 
						#on s'assure ensuite que l'URL est bien encodé en UTF-8
						#pour cela, on isole la valeur du charset de l'URL et on la stocke dans une variable
						#il faut récupérer le 2ème champ 
						encodageURL=$(curl -I $ligne | egrep -i "charset" | cut -f2 -d= | tr [a-z] [A-Z] | tr -d "\r");
						
						#on teste la valeur de encodageURL
						#si l'encodage est bien UTF-8 : on peut récupérer la page aspirée et remplir le tableau
						if [[ $encodageURL == "UTF-8" ]]
							then
							
								# ************************************ CodeHttp 200 + encodage UTF-8 ---> TRAITEMENTS ************************************
								
								# 1. Dump
								#on récupère le contenun textuel de la page via la commande lynx
								#on exécute lynx sur la page aspirée, pas sur l'URL (car certains sites nous empêchent de récupérer leur contenu textuel avec lynx)
								#on stocke les dumps dans le répertoire DUMP-TEXT
								lynx -dump -nolist -assume_charset="UTF-8" -display_charset="UTF-8" ./PAGES-ASPIREES/"$compteurTableau-$compteur".html > ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt;
								
								# 2. Tokenisation (traitement supplémentaire pour les URLs en arabe uniquement)
								if [[ $fichier == "urls_3_arabe.txt" ]]
									then
										java -cp ./PROGRAMMES/stanford-segmenter/stanford-segmenter-4.2.0.jar -mx1g edu.stanford.nlp.international.arabic.process.ArabicSegmenter -loadClassifier ./PROGRAMMES/stanford-segmenter/data/arabic-segmenter-atb+bn+arztrain.ser.gz -textFile ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt > ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8-tk.txt
										mv ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8-tk.txt ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt
								fi;
								
								# 3. Nombre d'occurrences de notre motif
								#on stocke ce nombre dans la variable compteur_motif
								compteur_motif=$(egrep -o -i $motif ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt | wc -l);
								
								# 4. Contextes avec egrep
								#Méthode 1 : egrep : construire des morceaux de corpus
								egrep -C 2 -i "$motif" ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt > ./CONTEXTES/"$compteurTableau-$compteur".txt;
								
								# 5. Contextes HTML avec minigrep
								perl ./PROGRAMMES/minigrepmultilingue/minigrepmultilingue.pl "UTF-8" ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt ./PROGRAMMES/minigrepmultilingue/motif-regexp.txt;
									#3 arguments du programme minigrepmultilingue.pl : l'encodage + le fichier + le motif
									#Attention le motif à chercher doit être indiqué dans le fichier motif-regexp.txt sous forme d'expression régulière !
								#Attention il faut "sauvegarder" le résultat
								mv resultat-extraction.html ./CONTEXTES/"$compteurTableau-$compteur".html;
								
								# 6. Index hiérarchique
								egrep -i -o "\w+" ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt | sort | uniq -c | sort -n -r -s -k 1,1 > ./DUMP-TEXT/"$compteurTableau-$compteur"-index.txt;
								
								# 7. Bigrammes (séquence de 2 mots consécutifs)
								tr " " "\n " < ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt | tr -s "\n" | egrep -v "^$" > index1.txt;
								tail -n +2 index1.txt > index2.txt;
								paste index1.txt index2.txt | sort | uniq -c | sort -r -n -s -k 1,1 > ./DUMP-TEXT/"$compteurTableau-$compteur"-bigramme.txt
								
								# ------------> On peut remplir toutes les lignes du tableau
								echo "<tr><td>$compteur</td>
								<td>$codeHttp</td>
								<td>$encodageURL</td>
								<td><a target=\"_blank\" href=\"$ligne\">$ligne</a></td>
								<td><a target=\"_blank\" href=\"../PAGES-ASPIREES/$compteurTableau-$compteur.html\">Page aspirée $compteur</a></td>
								<td><a target=\"_blank\" href=\"../DUMP-TEXT/$compteurTableau-$compteur-dump-utf8.txt\">Dump $compteur</a></td>
								<td>$compteur_motif</td>
								<td><a target=\"_blank\" href=\"../CONTEXTES/$compteurTableau-$compteur.txt\">Contexte txt $compteur</a></td>
								<td><a target=\"_blank\" href=\"../CONTEXTES/$compteurTableau-$compteur.html\">Contexte html $compteur</a></td>
								<td><a target=\"_blank\" href=\"../DUMP-TEXT/$compteurTableau-$compteur-index.txt\">Index $compteur</a></td>
								<td><a target=\"_blank\" href=\"../DUMP-TEXT/$compteurTableau-$compteur-bigramme.txt\">Bigramme $compteur</a></td>
								</tr>" >> $monTableau;
							
							else
							#code http = 200 OK mais encodage PAS UTF-8
														
							#est ce que curl a récupéré quelque chose ?
								#si curl a récupéré quelque chose (une chaîne non-vide), il faut vérifier si cette chaîne de caractère récupérée est effectivement un bon encodage
								if [[ $encodageURL != "" ]]
									then
										#on utilise iconv, l'option -l (--list) renvoie la liste de tous les encodages connus
										response=$(iconv -l | egrep "\b$encodageURL\b");
										if [[ $response != "" ]]
										#si l'encodage est bien dans la liste iconv
										
										# *************************** CodeHttp 200 + encodage récupéré par curl ---> TRAITEMENTS *****************************

											then
												# 1. Dump
												# Attention on effectue le lynx dans la valeur de l'encodage
												lynx -dump -nolist -assume_charset="$encodageURL" -display_charset="$encodageURL" ./PAGES-ASPIREES/"$compteurTableau-$compteur".html > ./DUMP-TEXT/"$compteurTableau-$compteur"-dump.txt;
												
												# 1bis. Conversion du dump en UTF-8
												iconv -f $encodageURL -t UTF-8 ./DUMP-TEXT/"$compteurTableau-$compteur"-dump.txt > ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt;
												
												# 2. Tokenisation (traitement supplémentaire pour les URLs en arabe uniquement)
												if [[ $fichier == "urls_3_arabe.txt" ]]
													then
														java -cp ./PROGRAMMES/stanford-segmenter/stanford-segmenter-4.2.0.jar -mx1g edu.stanford.nlp.international.arabic.process.ArabicSegmenter -loadClassifier ./PROGRAMMES/stanford-segmenter/data/arabic-segmenter-atb+bn+arztrain.ser.gz -textFile ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt > ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8-tk.txt;
														mv ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8-tk.txt ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt;
												fi;
												
												# 3. Nombre d'occurrences de notre motif
												compteur_motif=$(egrep -o -i $motif ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt | wc -l);
												
												# 4. Contextes avec egrep
												egrep -C 2 -i "$motif" ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt > ./CONTEXTES/"$compteurTableau-$compteur".txt;
												
												# 5. Contextes HTML avec minigrep
												perl ./PROGRAMMES/minigrepmultilingue/minigrepmultilingue.pl "UTF-8" ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt ./PROGRAMMES/minigrepmultilingue/motif-regexp.txt;
												mv resultat-extraction.html ./CONTEXTES/"$compteurTableau-$compteur".html;
												
												# 6. Index hiérarchique
												egrep -i -o "\w+" ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt | sort | uniq -c | sort -n -r -s -k 1,1 > ./DUMP-TEXT/"$compteurTableau-$compteur"-index.txt;
												
												# 7. Bigrammes (séquence de 2 mots consécutifs)
												tr " " "\n " < ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt | tr -s "\n" | egrep -v "^$" > index1.txt;
												tail -n +2 index1.txt > index2.txt;
												paste index1.txt index2.txt | sort | uniq -c | sort -r -n -s -k 1,1 > ./DUMP-TEXT/"$compteurTableau-$compteur"-bigramme.txt;
												
												# ------------> On peut remplir toutes les lignes du tableau
												echo "<tr><td>$compteur</td>
												<td>$codeHttp</td>
												<td bgcolor=#FDF08D>$encodageURL (détecté avec curl)</td>
												<td><a target=\"_blank\" href=\"$ligne\">$ligne</a></td>
												<td><a target=\"_blank\" href=\"../PAGES-ASPIREES/$compteurTableau-$compteur.html\">Page aspirée $compteur</a></td>
												<td><a target=\"_blank\" href=\"../DUMP-TEXT/$compteurTableau-$compteur-dump-utf8.txt\">Dump $compteur</a></td>
												<td>$compteur_motif</td>
												<td><a target=\"_blank\" href=\"../CONTEXTES/$compteurTableau-$compteur.txt\">Contexte txt $compteur</a></td>
												<td><a target=\"_blank\" href=\"../CONTEXTES/$compteurTableau-$compteur.html\">Contexte html $compteur</a></td>
												<td><a target=\"_blank\" href=\"../DUMP-TEXT/$compteurTableau-$compteur-index.txt\">Index $compteur</a></td>
												<td><a target=\"_blank\" href=\"../DUMP-TEXT/$compteurTableau-$compteur-bigramme.txt\">Bigramme $compteur</a></td>
												</tr>" >> $monTableau;
											
											else
												# ------------> PROBLEME - encodage détecté par curl non connu
												echo "<tr><td>$compteur</td>
												<td>$codeHttp</td>
												<td bgcolor=#E5AB55>$encodageURL (encodage non connu d'iconv)</td>
												<td><a target=\"_blank\" href=\"$ligne\">$ligne</a></td>
												<td>-</td>
												<td>-</td>
												<td>-</td>
												<td>-</td>
												<td>-</td>	
												<td>-</td>
												<td>-</td></tr>" >> $monTableau;
												
										fi;
										
									#si curl a récupéré une chaîne vide ($encodageURL == "") : il faut détecter l'encodage d'une autre manière : on utilise le programme miracle detect-encodage
									else 
										encodageExtrait=$(perl ./PROGRAMMES/detect-encoding.pl ./PAGES-ASPIREES/"$compteurTableau-$compteur".html | tr -d "\n" | tr -d "\r");
										#on s'assure que la valeur récupérée par le programme est effectivement un bon encodage
										#on procède de la même manière que lorsque l'encodage récupéré par curl n'est pas UTF-8 : avec iconv
										response=$(iconv -l | egrep "\b$encodageExtrait\b");
										if [[ $response != "" ]]
										#si l'encodage est bien dans la liste iconv
										
										# *************************** CodeHttp 200 + encodage récupéré par detect ---> TRAITEMENTS *****************************

											then
												# 1. Dump
												# Attention on effectue le lynx dans la valeur de l'encodage
												lynx -dump -nolist -assume_charset="$encodageExtrait" -display_charset="$encodageExtrait" ./PAGES-ASPIREES/"$compteurTableau-$compteur".html > ./DUMP-TEXT/"$compteurTableau-$compteur"-dump.txt;
												
												# 1bis. Conversion du dump en UTF-8
												iconv -f $encodageExtrait -t UTF-8 ./DUMP-TEXT/"$compteurTableau-$compteur"-dump.txt > ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt;
												
												# 2. Tokenisation (traitement supplémentaire pour les URLs en arabe uniquement)
												if [[ $fichier == "urls_3_arabe.txt" ]]
													then
														java -cp ./PROGRAMMES/stanford-segmenter/stanford-segmenter-4.2.0.jar -mx1g edu.stanford.nlp.international.arabic.process.ArabicSegmenter -loadClassifier ./PROGRAMMES/stanford-segmenter/data/arabic-segmenter-atb+bn+arztrain.ser.gz -textFile ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt > ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8-tk.txt;
														mv ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8-tk.txt ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt;
												fi;
												
												# 3. Nombre d'occurrences de notre motif
												compteur_motif=$(egrep -o -i $motif ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt | wc -l);
												
												# 4. Contextes avec egrep
												egrep -C 2 -i "$motif" ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt > ./CONTEXTES/"$compteurTableau-$compteur".txt;
												
												# 5. Contextes HTML avec minigrep
												perl ./PROGRAMMES/minigrepmultilingue/minigrepmultilingue.pl "UTF-8" ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt ./PROGRAMMES/minigrepmultilingue/motif-regexp.txt;
												mv resultat-extraction.html ./CONTEXTES/"$compteurTableau-$compteur".html;
												
												# 6. Index hiérarchique
												egrep -i -o "\w+" ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt | sort | uniq -c | sort -n -r -s -k 1,1 > ./DUMP-TEXT/"$compteurTableau-$compteur"-index.txt;
												
												# 7. Bigrammes (séquence de 2 mots consécutifs)
												tr " " "\n " < ./DUMP-TEXT/"$compteurTableau-$compteur"-dump-utf8.txt | tr -s "\n" | egrep -v "^$" > index1.txt;
												tail -n +2 index1.txt > index2.txt;
												paste index1.txt index2.txt | sort | uniq -c | sort -r -n -s -k 1,1 > ./DUMP-TEXT/"$compteurTableau-$compteur"-bigramme.txt;
												
												# ------------> On peut remplir toutes les lignes du tableau
												echo "<tr><td>$compteur</td>
												<td>$codeHttp</td>
												<td bgcolor=#FDF08D>$encodageExtrait (détecté avec detect-encoding)</td>
												<td><a target=\"_blank\" href=\"$ligne\">$ligne</a></td>
												<td><a target=\"_blank\" href=\"../PAGES-ASPIREES/$compteurTableau-$compteur.html\">Page aspirée $compteur</a></td>
												<td><a target=\"_blank\" href=\"../DUMP-TEXT/$compteurTableau-$compteur-dump-utf8.txt\">Dump $compteur</a></td>
												<td>$compteur_motif</td>
												<td><a target=\"_blank\" href=\"../CONTEXTES/$compteurTableau-$compteur.txt\">Contexte txt $compteur</a></td>
												<td><a target=\"_blank\" href=\"../CONTEXTES/$compteurTableau-$compteur.html\">Contexte html $compteur</a></td>
												<td><a target=\"_blank\" href=\"../DUMP-TEXT/$compteurTableau-$compteur-index.txt\">Index $compteur</a></td>
												<td><a target=\"_blank\" href=\"../DUMP-TEXT/$compteurTableau-$compteur-bigramme.txt\">Bigramme $compteur</a></td>
												</tr>" >> $monTableau;
											
											else
												# ------------> PROBLEME - encodage détecté par detect-encoding non connu
												echo "<tr><td>$compteur</td>
												<td>$codeHttp</td>
												<td bgcolor=#E5AB55>$encodageExtrait (encodage non connu d'iconv)</td>
												<td><a target=\"_blank\" href=\"$ligne\">$ligne</a></td>
												<td>-</td>
												<td>-</td>
												<td>-</td>
												<td>-</td>
												<td>-</td>	
												<td>-</td>
												<td>-</td></tr>" >> $monTableau;
										fi;
								fi;
						fi;
						
					else
					# PROBLEME de code http ---> On ne récupère aucune donnée de l'URL :-(
						echo "<tr><td>$compteur</td>
						<td bgcolor=#E5AB55>$codeHttp</td>
						<td>-</td>
						<td><a target=\"_blank\" href=\"$ligne\">$ligne</a></td>
						<td>-</td>
						<td>-</td>
						<td>-</td>
						<td>-</td>
						<td>-</td>	
						<td>-</td>
						<td>-</td></tr>" >> $monTableau;
				fi;
				
				compteur=$((compteur + 1)); ##incrémentation --> lecture de la prochaine ligne d'URL
			done < $dossierURL/$fichier 	#pour accéder au fichier input qui se trouve dans le sous-dossier, il faut redonner le chemin 
		echo "</table>" >> $monTableau; #fin du tableau
		echo "<p><hr color=\"blue\"></p>" >> $monTableau
		compteurTableau=$((compteurTableau + 1)) #incrémentation --> prochain fichier txt, prochain tableau
	done;
echo "</body></html>" >> $monTableau; #fin de la page HTML
#fin du programme